######################################################################################
# SCRIPT:      Insert-x-ms-segment-and-label-it-from-menu_ES.praat                   #
# LANGUAGE:    Spanish                                                               #     
# BASED ON:    !S_Insert-ENTORNO_Select-x-ms+LabelTierSegment.praat                  #
# DATE:        02 May 2010                                                           #
# VERSION:     1.0                                                                   #
# DESCRIPTION: Creates a segment of a user-defined duration around the current       #     
#              cursor position, and lets user add a label to the segment by          #
#              selecting it from a menu.                                             #
# AUTHOR:      Scott Sadowsky < s s a d o w s k y @ g m a i l . c o m >              #
# THANKS:      GetTierName procedure adapted from a script by Mietta Lennes.         #
# FILE FORMAT: DOS/Windows - ANSI as UTF-8. You can indeed use Unicode IPA           #
#              symbols, such as [t̠͡ʃ], [ĩ] and [ɣ̞], in your menu                        #
######################################################################################

############################################################################
# Copyright © 2010 by Scott Sadowsky                                       #
# <ssadowsky ARROBA gmail DOT com>                                         #
#                                                                          #
# Licensed under the GNU General Public License v3                         #
#                                                                          #
# This program is free software: you can redistribute it and/or modify     #
# it under the terms of the GNU General Public License as published by     #
# the Free Software Foundation, either version 3 of the License, or        #
# (at your option) any later version.                                      #
#                                                                          #
# This program is distributed in the hope that it will be useful,          #
# but WITHOUT ANY WARRANTY; without even the implied warranty of           #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
# GNU General Public License for more details.                             #
#                                                                          #
# You should have received a copy of the GNU General Public License        #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.	#
############################################################################

# Dump selected TextGrid name to variable <soundName>
soundName$ = selected$ ("TextGrid", 1)

# Ask user to select environment to be inserted as interval label.
form Crear segmento de x ms y agregar etiqueta de un listado
	comment ACCIÓN: Este script crea un segmento de la duración que elijas en torno al cursor. 
	comment Luego, permite que elijas de una lista la etiqueta que quieres colocar en el segmento.
	comment REQUISITO: Seleccionaro sólo el TextGrid en la lista de objetos de Praat.
	comment 
	sentence Nombre_del_tier Coloca en el script el nombre del tier que más usas
	positive Duracion_del_intervalo_(ms) 50

	# In the list of options that follows, the first argument of each "option" 
	# line must be separated from the second argument by one or more SPACES. Tabs
	# will not work due to the chomp routine used later on.
	comment 
	optionmenu Etiqueta: 1
	
	############### A PARTIR DE AQUÍ PUEDES PERSONALZAR LA LISTA DE OPCIONES ###############
	option Texto-a-insertar   Descripción amigable
	option ------ ENTORNOS ------
	option #_     Posición inicial
	option V_V   Posición intervocálica
	option _#   Posición final
	option ------ FONEMAS ------
	option m    bilabial nasal sonora
	option n     lámino-alveolar nasal sonora
	option ɲ     prepalatal nasal sonora
	option b     bilabial oclusiva sonora
	option d̪     postdental oclusiva sonora
	option g     velar oclusiva sonora
	option p     bilabial oclusiva áfona
	option t̪     postdental oclusiva áfona
	option k     velar oclusiva áfona
	option t̠͡ʃ    post-alveolar africada áfona
	option s     dorso-alveolar fricativa áfona
	option f     labiodental fricativa áfona
	option j     prepalatal aproximante sonora
	option w   velar aproximante labializada sonora
	option ɾ     vibrante simple sonora
	option r     vibrante múltiple sonora
	option l      lámino-alveolar lateral sonora
	option ------ ALÓFONOS ------
	option t̪ˢ     post-dental africada áfona
	option t̠͡ʃ    post-alveolar africada áfona
	option ʃ      post-alveolar fricativa áfona
	option ɟ     palatal oclusiva sonora
	option c     palatal oclusiva áfona
	option ɣ̟    palatal fricativa sonora
	option ç     palatal fricativa áfona
	option d̪    postdental oclusiva sonora
	option ð     postdental fricativa sonora
	option ð̞    postdental aproximante sonora
	option ...   etcétera
############################## FIN DE PERSONALIZACIÓN ##############################

endform



# Set or calculate variables and get info on the selection.
	# Convert windowSize from SEC to MS and divide by 2 to get cursorShift value for later use.
	cursorShift = (duracion_del_intervalo/1000)/2

	# Select  window around cursor. Variable <cursor> holds current position.
	cursor = Get cursor
	Select... cursor-cursorShift cursor+cursorShift

	# Store values of beginning and end of interval for future use.
	selectionStart = Get start of selection
	selectionEnd = Get end of selection


# Remove the part of the option menu string after the first space (i.e. the description)
stringCutoff = index (etiqueta$, " ")
stringEnd = stringCutoff - 1

# Set the interval text string to the phonetic environment string minus the descriptive
# bits after the first space
intervalText$ = left$ (etiqueta$, stringEnd)


# Only for debugging purposes. Comment out when using script for real
# printline 'etiqueta$'
# printline 'intervalText$'

# Leave the editor. Just gotta do it.
endeditor

call insertLabel
select Sound 'soundName$'
	plus TextGrid 'soundName$'
exit


##############################################################################
# PROCEDURE:	insertLabel
# AUTHOR:		Scott Sadowsky
# DESCRIPTION:	Inserts the selected allophone of /s/ code and the optional 
#				word or words in the newly-created interval on the appropriate tier.
##############################################################################
procedure insertLabel

	# Get tier number from name
	call GetTierNum 'nombre_del_tier$' tierNum
	
	# Select the TextGrid in order to make subsequent parts possible.
	select TextGrid 'soundName$'
	
	# ERROR CHECKING
	# Determine whether the current selection is valid, is adjacent to other
	# intervals on the left, right, or both, or overlaps other intervals.

		# Get the interval numbers of the start and end points of the current
		# selection, as well as of points just a wee bit before and after each.
		startIntervalNum = Get interval at time... tierNum selectionStart
		endIntervalNum = Get interval at time... tierNum selectionEnd
		minusStartIntervalNum = Get interval at time... tierNum selectionStart-0.00001
		minusEndIntervalNum = Get interval at time... tierNum selectionEnd-0.00001
		plusStartIntervalNum = Get interval at time... tierNum selectionStart+0.00001
		plusEndIntervalNum = Get interval at time... tierNum selectionEnd+0.00001

		# Use the above values to determine the status of the current selection.
		# In the case of overlap or invalid input, kill the script immediately.
		if ( startIntervalNum = endIntervalNum ) and ( minusStartIntervalNum =  minusEndIntervalNum )
			currentSelection$ = "OK"
			insertLeft = 1
			insertRight = 1
			
		elsif ( startIntervalNum = endIntervalNum ) and ( minusStartIntervalNum <  minusEndIntervalNum )
			currentSelection$ = "LeftAdjacent"
			insertLeft = 0
			insertRight = 1
			
		elsif ( startIntervalNum < endIntervalNum ) and ( minusStartIntervalNum = minusEndIntervalNum )
			currentSelection$ = "RightAdjacent"
			insertLeft = 1
			insertRight = 0
			
		elsif ( startIntervalNum < endIntervalNum ) and ( startIntervalNum > minusStartIntervalNum )
			currentSelection$ = "DoubleAdjacent"
			insertLeft = 0
			insertRight = 0
			
		elsif ( startIntervalNum < endIntervalNum ) and ( startIntervalNum = minusStartIntervalNum )
			currentSelection$ = "Overlap"
			exit ¡ERROR!'newline$''newline$'No se puede crear un intervalo que traslape
				... otro(s).'newline$''newline$'Cierra esta ventana (pulsando <OK>) y la ventana
				... del script (pulsando <CANCEL>) e intenta de nuevo.'newline$'
			
		else currentSelection$ = "INVALID INPUT"
			exit Congratulations - you somehow managed to break this script!'newline$''newline$'
		
		endif
		
		##### FOR DEBUGGING #####
		#clearinfo
		#printline startIntervalNum = 'startIntervalNum'
		#printline endIntervalNum = 'endIntervalNum'
		#printline minusStartIntervalNum = 'minusStartIntervalNum'
		#printline minusEndIntervalNum = 'minusEndIntervalNum'
		#printline plusStartIntervalNum = 'plusStartIntervalNum'
		#printline plusEndIntervalNum = 'plusEndIntervalNum'
		#printline RESULT: 'currentSelection$'
		
	# Insert tier interval boundaries where appropriate.
	if ( insertLeft = 1 )
		Insert boundary... tierNum selectionStart
	endif
	
	if ( insertRight = 1 )
		Insert boundary... tierNum selectionEnd
	endif
	
	# Get the number of the interval at cursor position.
	intervalNum = Get interval at time... tierNum cursor

	# Insert the desired text in the chosen tier and interval.
	Set interval text... tierNum intervalNum 'intervalText$'

endproc



##############################################################################
# PROCEDURE:	GetTierNum .name$ .variable$
# DESCRIPTION:	Finds the number of a tier that has a given label.
# WHERE:	<'tierName$'> is the variable which holds the name of the desired tier.
#		<tierNum> is the *name* of the numeric variable which will be passed to 
#		  the procedure and assigned the result value there.
# GLOBAL VARIABLES NEEDED:
#	<soundName$> is the name of the sound and TextGrid file being used.
# SAMPLE SCRIPT TO CALL PROCEDURE:
#	soundName$ = selected$ ("TextGrid", 1)
#	tierName$ = "phones2"
#	call GetTierNum 'tierName$' tierNum
#	clearinfo
#	printline The 'tierName$' tier is number 'tierNum'.
# THANKS: Adapted from a script by Mietta Lennes.
##############################################################################
procedure GetTierNum .name$ .variable$
	
	select TextGrid 'soundName$'
	.numberOfTiers = Get number of tiers
	
	# Cycle through the tiers in the TextGrid and check tier names until the 
	# desired one is found or all tiers have been tried unsuccessfully.
	.itier = 1
	repeat
		.currentTier$ = Get tier name... .itier
		.itier = .itier + 1
	until .currentTier$ = .name$ or .itier > .numberOfTiers

	# If no tier has the name being searched for, set the variable passed back 
	# to the main part of the script (whose name is contained in .variable$) to 0. 
	if .currentTier$ <> .name$
		'.variable$' = 0
	
	# If the tier being searched for WAS found, set the variable passed as the
	# procedure's second parameter (held in .variable$) to the tier number.
	else
		'.variable$' = .itier - 1
	endif

	# If the tier being searched for was not found, die and throw an error message.
	if '.variable$' = 0
		exit No existe ningún tier llamado '.name$' en el archivo 'soundName$'!
	endif

endproc
